@@ -5,10 +5,10 @@ knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
 library(openEBGM)
 data(caers)
 
-processed <- processRaw(data = caers)
+processed <- processRaw(caers)
 processed[1:4, 1:4]
 
-squashed <- squashData(data = processed) #Using defaults
+squashed <- squashData(processed) #Using defaults
 head(squashed)
 
 nrow(processed)
@@ -48,3 +48,25 @@ autoHyper(squashed2, theta_init = theta_init, conf_ints = TRUE)$conf_int
 ## ----hyperparameter estimation example, warning = FALSE------------------
 exploreHypers(data = squashed2, theta_init = theta_init, std_errors = TRUE)
 
+## ----hyperEM example, warning = FALSE------------------------------------
+data(caers)
+proc <- processRaw(caers)
+squashed <- squashData(proc, bin_size = 100, keep_bins = 0)
+squashed <- squashData(squashed, count = 2, bin_size = 12)
+hyperEM_ests <- hyperEM(squashed, theta_init_vec = c(1, 1, 2, 2, .3),
+                        conf_int = TRUE, track = TRUE)
+str(hyperEM_ests)
+
+## ----hyperEM plotting example, warning = FALSE, fig.width = 7, fig.height = 10----
+library(ggplot2)
+library(tidyr)
+pdat <- gather(hyperEM_ests$tracking, key = "metric", value = "value", logL:P)
+pdat$metric <- factor(pdat$metric, levels = unique(pdat$metric), ordered = TRUE)
+ggplot(pdat, aes(x = iter, y = value)) +
+  geom_line(size = 1.1, col = "blue") +
+  facet_grid(metric ~ ., scales = "free") +
+  ggtitle("Convergence Assessment",
+          sub = "Dashed red line indicates accelerated estimate") +
+  labs(x = "Iteration Count", y = "Estimate") +
+  geom_vline(xintercept = c(100, 200), size = 1, linetype = 2, col = "red")
+
